package org.grails.plugins.springsecurity.service.acl

import org.springframework.security.acls.jdbc.LookupStrategy
import org.springframework.security.acls.objectidentity.ObjectIdentity
import org.springframework.security.acls.sid.Sid
import org.springframework.security.acls.jdbc.AclCache
import org.springframework.security.acls.domain.AclAuthorizationStrategy
import org.springframework.security.acls.domain.AclImpl
import org.springframework.security.acls.domain.AuditLogger
import org.springframework.security.acls.domain.ConsoleAuditLogger
import org.springframework.security.acls.sid.GrantedAuthoritySid
import org.springframework.security.acls.sid.PrincipalSid
import org.springframework.security.acls.domain.AccessControlEntryImpl
import org.springframework.security.acls.domain.BasePermission

/*
 *  @author Stephan February
 *  @author <a href='mailto:phillip.sky@gmx.de'>Phillip Merensky</a>
*/
class LookupStrategyService implements LookupStrategy {

	static transactional = true

	//private DataSource dataSource
	def aclCache
	def aclAuthorizationStrategy

	private auditLogger = new ConsoleAuditLogger()

	/*
	 * @param identities
	 * @param sids This information is currently not interpreted.Thus, information for all sids will be returned.
	 */
	Map readAclsById(ObjectIdentity[] identities, Sid[] sids) {

		//we currently do not use the sids information
		//Sql sql = new Sql(dataSource)
		def retMap = [:]

		for (identity in identities) {

			String javaType = identity.javaType.name
			long domainObjId = identity.identifier.longValue()

			log.debug("ACL: javaType is $javaType")
			log.debug("ACL: domainObjId is $domainObjId")

			def c = AclObjectIdentity.createCriteria()
			def objectIdentityList = c.list {
				and {
					eq("objectIdIdentity", domainObjId)
					objectIdClass {
						eq("cLASS", javaType)
					}
				}
			}

			log.debug("ACL: criteria completed")

			if (objectIdentityList) {
				def objectIdentity = objectIdentityList[0]
				log.debug("ACL: objectIdentityId is $objectIdentity.objectIdIdentity")

				def aclSid
				if (objectIdentity.ownerSid.principal) {
					aclSid = new PrincipalSid(objectIdentity.ownerSid.sid)  //e.g. stephan
				}
				else {
					aclSid = new GrantedAuthoritySid(objectIdentity.ownerSid.sid) //e.g. ROLE_USER
				}

				AclImpl aclImpl = new AclImpl(identity, objectIdentity.id,
						aclAuthorizationStrategy, auditLogger, objectIdentity.parentObject,
						null, objectIdentity.entriesInheriting, aclSid)

				//add all the ACEs
				for (entry in objectIdentity.aclEntries) {
					log.debug "ACL: entrysid is $entry.sid.sid"
					def sid
					if (entry.sid.principal) {
						sid = new PrincipalSid(entry.sid.sid)  //e.g. stephan
					}
					else {
						sid = new GrantedAuthoritySid(entry.sid.sid) //e.g. ROLE_USER
					}

					def permission = BasePermission.buildFromMask(entry.mask)

					def ace = new AccessControlEntryImpl(entry.id, aclImpl, sid,
							permission, entry.granting, entry.auditSuccess,
							entry.auditFailure)

					aclImpl.aces << ace
				}
				//If we do this in the loop before we will not get a result if
				//an ACL with no entries exists. This would prevent reinserting aces
				//into a yet existing acl
				retMap.put(identity, aclImpl)
			}
		}

		return retMap
	}
}
